; Proof of concept code for reading/writing ds1315
; rtc in Dragon rom socket
;
; This is for the specific case where the ds1315 is
; co-resident with the rom that contains the cpu vectors
; with no means of deselecting the rom.
;
; Should allow use of Mike Miller's ds1315 rtc board
; in a Dragon 32 with a suitable adaptor.
;
; This version pushes a code path onto the stack
; and then uses rti instructions to consume it.
; Requires 196 bytes of stack to write to the rtc.
; Reader only requires 37 bytes.
;
; Refer to Mike's documentation for further details.
;
; S.Orchard Dec 2021


m_rtc_wbit macro
    if (\1)
      lda ,y
    else
      lda ,x
    endif
    endm

m_rtc_wbyte macro
    m_rtc_wbit \1 & 1
    m_rtc_wbit \1 & 2
    m_rtc_wbit \1 & 4
    m_rtc_wbit \1 & 8
    m_rtc_wbit \1 & 16
    m_rtc_wbit \1 & 32
    m_rtc_wbit \1 & 64
    m_rtc_wbit \1 & 128
    endm


RTCBASE equ $a000
READBUF equ $400


    org $4000

    bra read_test

;--------------------------------------
; write to rtc
; pushes state onto the stack representing code path
; to be consumed by rti instructions
;
; each step in the code path is represented by a
; partial state save (cc,pc)
;
; requires 3*64 + 4 = 196 bytes of stack

write_test
    pshs cc             ; save caller's cc

    ldx #rtcdata+8      ; data to write to rtc (+8)
    ldd #$5008          ; a = cc value  b = 8 bytes
    stb count
    ldu #rtc_wr_done    ; push exit path first
    pshs a,u            ;

    ; encode the 64 bits of rtc data
wr1 ldb ,-x
    ldy #8              ; 8 bits
wr2 ldu #rtc_wr_0       ; routine to write 0
    lslb
    bcc wr3
    ldu #rtc_wr_1       ; routine to write 1
wr3 pshs a,u
    leay -1,y
    bne wr2             ; next bit
    dec count
    bne wr1             ; next byte

    orcc #$50

    bra run_routine

;--------------------------------------
; read from rtc
; pushes state onto the stack representing code path
; to be consumed by rti instructions
;
; most steps in the code path are represented by
; partial state saves (cc,pc)
;
; the initial entry however requires some registers to be
; set up, so this is represented by an entire state save
;
; requires 3*8 + 13 = 37 bytes of stack space

read_test

    pshs cc               ; save caller's cc

    ; push reader sequence onto stack for rti to process
    ; note these are partial state saves (cc,pc)

    ldd #$5008            ; a = ccr value  b = 8 loops
    ldu #rtc_rd_store7
rd1 pshs a,u
    leau -5,u             ; adjust to rtc_rd_store6 etc
    decb
    bne rd1

    ; set up entry params for reader
    ; and push them onto stack with entire state flag set

    ldu #rtc_rd_getbyte   ; entry point
    pshs u                ; push pc part of state

    ldx #RTCBASE+4        ; rtc base address for reading
    ldy #rtc_rd_store     ; loop address

    orcc #$d0             ; mask interrupts + set entire state
    pshs cc,dp,d,x,y,u    ; push rest of state

    ;bra run_routine
    ; (fall through to run_routine)

;--------------------------------------
; This starts the rtc read or write sequence
; It begins by running a hard-coded access sequence
; then transfers control to whatever is on the stack via rti

run_routine

    ; set up for access pattern write
    ldx #RTCBASE        ; address for rtc write 0
    leay 1,x            ; address for rtc write 1

    ; and off we go...

    ; write access pattern
    m_rtc_wbyte $c5
    m_rtc_wbyte $3a
    m_rtc_wbyte $a3
    m_rtc_wbyte $5c
    m_rtc_wbyte $c5
    m_rtc_wbyte $3a
    m_rtc_wbyte $a3
    m_rtc_wbyte $5c

    rti                 ; jump to reader or writer

;--------------------------------------
; reader subroutines

m_rtc_rbit macro
    lda ,x
    lsra
    rorb
    endm

; read a byte from rtc

rtc_rd_store
    stb ,u
rtc_rd_getbyte
    m_rtc_rbit
    m_rtc_rbit
    m_rtc_rbit
    m_rtc_rbit
    m_rtc_rbit
    m_rtc_rbit
    m_rtc_rbit
    m_rtc_rbit
    rti


m_rtc_rd_store macro
    ldu #READBUF + \1
    jmp ,y
    endm

; these code fragments store each byte from the rtc

rtc_rd_store0
    m_rtc_rd_store 0

rtc_rd_store1
    m_rtc_rd_store 1

rtc_rd_store2
    m_rtc_rd_store 2

rtc_rd_store3
    m_rtc_rd_store 3

rtc_rd_store4
    m_rtc_rd_store 4

rtc_rd_store5
    m_rtc_rd_store 5

rtc_rd_store6
    m_rtc_rd_store 6

rtc_rd_store7
    ; rtc reading complete
    stb READBUF+7
    puls cc,pc          ; return to caller


;--------------------------------------
; writer subroutines

; write a zero

rtc_wr_0
    lda ,x
    rti

; write a one

rtc_wr_1
    lda ,y
    rti

; exit

rtc_wr_done
    ; rtc write complete
    puls cc,pc      ; return to caller

;--------------------------------------

rtcdata     fcb 0,0,0,0,$15,$03,$12,$21

count   rmb 1
